/**
 * @file 通过 Schema 生成可以让用户输入的 Question Form
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _getIterator2 = require('babel-runtime/core-js/get-iterator');

var _getIterator3 = _interopRequireDefault(_getIterator2);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * 获取当前用户的 git 账号信息
 *
 * @return {Promise} promise 对象
 */
var getGitInfo = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
        var author, email;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        author = void 0;
                        email = void 0;
                        _context.prev = 2;
                        _context.next = 5;
                        return exec('git config --get user.name');

                    case 5:
                        author = _context.sent;
                        _context.next = 8;
                        return exec('git config --get user.email');

                    case 8:
                        email = _context.sent;
                        _context.next = 13;
                        break;

                    case 11:
                        _context.prev = 11;
                        _context.t0 = _context['catch'](2);

                    case 13:
                        author = author && author[0] && author[0].toString().trim();
                        email = email && email[0] && email[0].toString().trim();

                        return _context.abrupt('return', { author, email });

                    case 16:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this, [[2, 11]]);
    }));

    return function getGitInfo() {
        return _ref.apply(this, arguments);
    };
}();

/**
 * 询问 list 类型的参数 (多选或者单选)
 *
 * @param  {string} key    参数的 key
 * @param  {Object} schema schema 内容
 * @param  {Object} params 当前已有的参数
 * @return {Object}        question 需要的参数
 */


/**
 * 询问 input 类型的参数
 *
 * @param  {string} key    参数的 key
 * @param  {Object} schema schema 内容
 * @param  {Object} params 当前已有的参数
 * @return {Object}        question 需要的参数
 */
var questionInput = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(key, schema, params) {
        var con, name, invalidate, defaultVal, itemLocals, userInfo, reg;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        con = schema[key];
                        name = con.name, invalidate = con.invalidate;
                        defaultVal = con.default;
                        itemLocals = con.locals && con.locals[locals.LANG];


                        if (itemLocals) {
                            name = itemLocals.name || name;
                            defaultVal = itemLocals.default || defaultVal;
                            invalidate = itemLocals.invalidate || invalidate;
                        }

                        con.validate = function () {
                            return !!1;
                        };

                        // 如果输入项是 author 或者 email 的，尝试的去 git config 中拿默认的内容

                        if (!(key === 'author' || key === 'email')) {
                            _context2.next = 11;
                            break;
                        }

                        _context2.next = 9;
                        return getGitInfo();

                    case 9:
                        userInfo = _context2.sent;

                        defaultVal = userInfo[key] || con.default;

                    case 11:
                        if (key === 'dirPath') {
                            defaultVal = path.resolve(process.cwd(), con.default || '');
                            con.validate = function (value) {
                                var nowPath = path.resolve(process.cwd(), value || '');

                                if (!fs.existsSync(nowPath)) {
                                    return invalidate || locals.INPUT_INVALID;
                                }
                                return true;
                            };
                        }

                        if (con.regExp) {
                            reg = new RegExp(con.regExp);


                            con.validate = function (value) {
                                if (!reg.test(value)) {
                                    return invalidate || locals.INPUT_INVALID;
                                }
                                return true;
                            };
                        }

                        return _context2.abrupt('return', {
                            'type': con.type === 'password' ? 'password' : 'input',
                            'name': key,
                            'message': `${locals.PLEASE_INPUT}${name}: `,
                            'default': defaultVal,
                            'validate': con.validate
                        });

                    case 14:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this);
    }));

    return function questionInput(_x, _x2, _x3) {
        return _ref2.apply(this, arguments);
    };
}();

/**
 * 解析schme, 生成 form 表单
 *
 * @param  {Object} schema  传入的 schema 规则
 * @return {Object}         获取的 form 参数
 */


function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var inquirer = require('inquirer');
var exec = require('mz/child_process').exec;
var path = require('path');
var fs = require('fs-extra');
var os = require('os');

var log = require('../../lib/utils/log');
var locals = require('../../locals')();function questionList(key, schema, params) {
    var con = schema[key];
    var sourceList = [];
    var choiceList = [];
    var text = '';
    var valueList = [];
    var listName = con.name;
    var listLocals = con.locals && con.locals[locals.LANG];
    if (listLocals) {
        listName = listLocals.name;
    }

    if (!con.dependence) {
        sourceList = con.list;
    } else if (con.depLevel > 0) {

        // 表示是级联的操作
        var dependence = con.dependence;
        var ref = con.ref;
        var depList = schema[dependence].list;
        var depValue = params[dependence] || schema[dependence].list[0].value;

        depList.forEach(function (depItem) {
            if (depItem.value === depValue) {
                sourceList = depItem.subList && depItem.subList[ref] ? depItem.subList[ref] : [];
            }
        });
    }

    sourceList.forEach(function (item, index) {
        var url = '';
        var desc = item.desc,
            name = item.name;

        var itemLocals = item.locals && item.locals[locals.LANG];

        if (itemLocals) {
            desc = itemLocals.desc || desc;
            name = itemLocals.name || name;
        }

        desc = log.chalk.gray('\n    ' + desc);

        // if (item.url) {
        //     url = '\n    - ' + log.chalk.yellow.bold.underline(item.url);
        // }
        // else if (item.imgs && item.imgs[0]) {
        //     item.imgs.forEach(imgO => {
        //         let item = '\n    - '
        //             + log.chalk.yellow.bold.underline(imgO.src)
        //             + (imgO.alt ? ' - ' + imgO.alt : '');
        //         url += item;
        //     });
        // }
        // else if (item.img) {
        //     url = '\n    - ' + log.chalk.yellow.bold.underline(item.img);
        // }

        choiceList.push({
            value: item.value,
            name: `${name}${desc}${url}`,
            short: item.value
        });
        valueList.push(item.value);
        text += '' + log.chalk.blue('\n    [' + log.chalk.yellow(index + 1) + '] ' + name) + desc;
    });

    // 如果是 windows 下的 git bash 环境，由于没有交互 GUI，所以就采用文本输入的方式来解决
    if (os.platform() === 'win32' && process.env.ORIGINAL_PATH) {
        return {
            'type': 'input',
            'name': key,
            'message': locals.PLEASE_INPUT_NUM_DESC + ' ' + listName + '：' + text + '\n' + log.chalk.green('?') + ' ' + locals.PLEASE_INPUT_NUM + '：',
            'default': 1,
            'valueList': valueList,
            'validate'(value) {
                if (!/\d+/.test(value) || +value > valueList.length || +value <= 0) {
                    return locals.PLEASE_INPUT_RIGHR_NUM;
                }
                return true;
            }
        };
    }

    return {
        'type': 'list',
        'name': key,
        'message': `${locals.PLEASE_SELECT}${listName} (${log.chalk.green(locals.PLEASE_SELECT_DESC)}): `,
        'choices': choiceList,
        'default': choiceList[0].value || '',
        'checked': !!con.checkbox,
        'pageSize': 1000
    };
}

/**
 * 询问 boolean 类型的参数
 *
 * @param  {string} key    参数的 key
 * @param  {Object} schema schema 内容
 * @param  {Object} params 当前已有的参数
 * @return {Object}        question 需要的参数
 */
function questionYesOrNo(key, schema, params) {
    var con = schema[key];
    var name = con.name;
    var itemLocals = con.locals && con.locals[locals.LANG];

    if (itemLocals) {
        name = itemLocals.name || name;
    }

    return {
        'type': 'confirm',
        'name': key,
        'default': false,
        'message': `${name}? :`
    };
}module.exports = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(schema) {
        var params, _iteratorNormalCompletion, _didIteratorError, _iteratorError, _iterator, _step, key, con, type, opts, data;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        params = {};
                        _iteratorNormalCompletion = true;
                        _didIteratorError = false;
                        _iteratorError = undefined;
                        _context3.prev = 4;
                        _iterator = (0, _getIterator3.default)((0, _keys2.default)(schema));

                    case 6:
                        if (_iteratorNormalCompletion = (_step = _iterator.next()).done) {
                            _context3.next = 36;
                            break;
                        }

                        key = _step.value;
                        con = schema[key];
                        type = con.type;
                        opts = {};
                        data = {};
                        _context3.t0 = type;
                        _context3.next = _context3.t0 === 'string' ? 15 : _context3.t0 === 'number' ? 15 : _context3.t0 === 'password' ? 15 : _context3.t0 === 'boolean' ? 19 : _context3.t0 === 'list' ? 21 : 23;
                        break;

                    case 15:
                        _context3.next = 17;
                        return questionInput(key, schema, params);

                    case 17:
                        opts = _context3.sent;
                        return _context3.abrupt('break', 23);

                    case 19:
                        opts = questionYesOrNo(key, schema, params);
                        return _context3.abrupt('break', 23);

                    case 21:
                        opts = questionList(key, schema, params);
                        return _context3.abrupt('break', 23);

                    case 23:
                        if (!(type === 'list' && con.list.length === 1)) {
                            _context3.next = 27;
                            break;
                        }

                        data[key] = con.list[0].value;
                        _context3.next = 32;
                        break;

                    case 27:
                        if (con.disable) {
                            _context3.next = 32;
                            break;
                        }

                        _context3.next = 30;
                        return inquirer.prompt([opts]);

                    case 30:
                        data = _context3.sent;

                        if (opts.valueList) {
                            data[key] = opts.valueList[+data[key] - 1];
                        }

                    case 32:

                        params = (0, _assign2.default)({}, params, data);

                    case 33:
                        _iteratorNormalCompletion = true;
                        _context3.next = 6;
                        break;

                    case 36:
                        _context3.next = 42;
                        break;

                    case 38:
                        _context3.prev = 38;
                        _context3.t1 = _context3['catch'](4);
                        _didIteratorError = true;
                        _iteratorError = _context3.t1;

                    case 42:
                        _context3.prev = 42;
                        _context3.prev = 43;

                        if (!_iteratorNormalCompletion && _iterator.return) {
                            _iterator.return();
                        }

                    case 45:
                        _context3.prev = 45;

                        if (!_didIteratorError) {
                            _context3.next = 48;
                            break;
                        }

                        throw _iteratorError;

                    case 48:
                        return _context3.finish(45);

                    case 49:
                        return _context3.finish(42);

                    case 50:
                        return _context3.abrupt('return', params);

                    case 51:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this, [[4, 38, 42, 50], [43,, 45, 49]]);
    }));

    return function (_x4) {
        return _ref3.apply(this, arguments);
    };
}();