/**
 * @file server command for lavas
 * @author mj(zoumiaojiang@gmail.com)
 */

'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var path = require('path');
var fs = require('fs-extra');

var _require = require('child_process'),
    fork = _require.fork;

var express = require('express');
var app = express();
var historyMiddleware = require('connect-history-api-fallback');

var log = require('../../lib/utils/log');
var utils = require('../../lib/utils');
var locals = require('../../locals')();

var DEFAULT_LAVAS_CONFIG = 'lavas.config.js';
var DEFAULT_DEV_SERVER_SCRIPT = 'server.dev.js';
var DEFAULT_PROD_SERVER_SCRIPT = 'server.prod.js';

module.exports = function (program) {
    var _this = this;

    // lavas dev
    program.command('dev [config]').description(locals.START_DEV).option('-p, --port <port>', locals.START_PORT).action(function () {
        var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(config, _ref2) {
            var port = _ref2.port;
            var configPath, configJson, appDir, serverScriptPath, isServerScriptExist, options;
            return _regenerator2.default.wrap(function _callee$(_context) {
                while (1) {
                    switch (_context.prev = _context.next) {
                        case 0:
                            log.info(locals.START_DEV_SERVER + '...');

                            // find config file path
                            configPath = void 0;

                            if (config) {
                                if (!path.isAbsolute(config)) {
                                    configPath = path.resolve(process.cwd(), config);
                                }
                            } else {
                                configPath = path.resolve(utils.getLavasProjectRoot(), DEFAULT_LAVAS_CONFIG);
                            }

                            _context.next = 5;
                            return fs.pathExists(configPath);

                        case 5:
                            if (_context.sent) {
                                _context.next = 8;
                                break;
                            }

                            log.warn(`${locals.START_NO_FILE} ${configPath}`);
                            return _context.abrupt('return');

                        case 8:

                            // read appDir from config file
                            configJson = void 0;

                            try {
                                configJson = require(configPath);
                            } catch (e) {
                                log.warn(`${locals.INPUT_INVALID} ${configPath}`);
                            }

                            appDir = configJson.appDir || utils.getLavasProjectRoot();
                            serverScriptPath = path.resolve(appDir, DEFAULT_DEV_SERVER_SCRIPT);
                            _context.next = 14;
                            return fs.pathExists(serverScriptPath);

                        case 14:
                            isServerScriptExist = _context.sent;

                            if (isServerScriptExist) {
                                _context.next = 18;
                                break;
                            }

                            log.warn(`${locals.START_NO_FILE} ${serverScriptPath}`);
                            return _context.abrupt('return');

                        case 18:

                            process.env.NODE_ENV = 'development';

                            if (port) {
                                process.env.PORT = Number(port);
                            }

                            options = [];

                            if (config) {
                                options.push(configPath);
                            }
                            fork(serverScriptPath, options);

                        case 23:
                        case 'end':
                            return _context.stop();
                    }
                }
            }, _callee, _this);
        }));

        return function (_x, _x2) {
            return _ref.apply(this, arguments);
        };
    }());

    // lavas start
    program.command('start').description(locals.START_PROD).option('-p, --port <port>', locals.START_PORT).action(function () {
        var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(_ref4) {
            var port = _ref4.port;
            var serverScriptPath, isServerScriptExist;
            return _regenerator2.default.wrap(function _callee2$(_context2) {
                while (1) {
                    switch (_context2.prev = _context2.next) {
                        case 0:
                            serverScriptPath = path.resolve(utils.getLavasProjectRoot(), DEFAULT_PROD_SERVER_SCRIPT);
                            _context2.next = 3;
                            return fs.pathExists(serverScriptPath);

                        case 3:
                            isServerScriptExist = _context2.sent;

                            if (isServerScriptExist) {
                                _context2.next = 7;
                                break;
                            }

                            log.warn(`${locals.START_NO_FILE} ${serverScriptPath}`);
                            return _context2.abrupt('return');

                        case 7:

                            process.env.NODE_ENV = 'production';

                            if (port) {
                                process.env.PORT = Number(port);
                            }

                            fork(serverScriptPath);

                        case 10:
                        case 'end':
                            return _context2.stop();
                    }
                }
            }, _callee2, _this);
        }));

        return function (_x3) {
            return _ref3.apply(this, arguments);
        };
    }());

    // lavas static
    program.command('static').description(locals.START_STATIC).option('-p, --port <port>', locals.START_PORT).action(function () {
        var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(_ref6) {
            var port = _ref6.port;
            var routesJsonPath, baseUrl;
            return _regenerator2.default.wrap(function _callee3$(_context3) {
                while (1) {
                    switch (_context3.prev = _context3.next) {
                        case 0:
                            log.info(locals.START_STATIC + '...');

                            routesJsonPath = path.resolve(process.cwd(), 'lavas/routes.json');

                            port = port || 8080;

                            // start static server with lavas routes configured
                            _context3.next = 5;
                            return fs.pathExists(routesJsonPath);

                        case 5:
                            if (!_context3.sent) {
                                _context3.next = 9;
                                break;
                            }

                            try {
                                baseUrl = require(routesJsonPath).base;

                                if (!baseUrl || baseUrl === '/') {
                                    // redirect all requests to '/index.html'
                                    app.use(historyMiddleware({
                                        htmlAcceptHeaders: ['text/html'],
                                        disableDotRule: false // ignore paths with dot inside
                                    }));

                                    app.use(express.static('.'));
                                } else {
                                    // fix trailing '/'
                                    // @see https://lavas.baidu.com/guide/v2/advanced/multi-lavas#express-%E5%A4%84%E7%90%86-spa-%E8%B7%AF%E7%94%B1%E7%9A%84%E5%B0%8F%E9%97%AE%E9%A2%98-%E6%89%A9%E5%B1%95
                                    if (!baseUrl.endsWith('/')) {
                                        baseUrl += '/';
                                    }

                                    app.use('/', function (req, res, next) {
                                        var requestUrl = req.url.replace(/\?.+?$/, '');

                                        if (requestUrl === baseUrl.substring(0, baseUrl.length - 1)) {
                                            req.url = requestUrl + '/';
                                        }

                                        next();
                                    });

                                    app.use(baseUrl, historyMiddleware({
                                        htmlAcceptHeaders: ['text/html'],
                                        disableDotRule: false // ignore paths with dot inside
                                    }));

                                    app.use(baseUrl, express.static('.'));
                                }
                            } catch (e) {}
                            _context3.next = 10;
                            break;

                        case 9:
                            app.use(express.static('.'));

                        case 10:

                            app.listen(port, function () {
                                return log.info(`Static server start at localhost:${port}`);
                            });

                        case 11:
                        case 'end':
                            return _context3.stop();
                    }
                }
            }, _callee3, _this);
        }));

        return function (_x4) {
            return _ref5.apply(this, arguments);
        };
    }());
};