'use strict';

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @file schema modules
 * @author mj(zoumiaojiang@gmail.com)
 */

var getMeta = require('./getMeta');
var store = require('./store');

/**
 * 把约定的 JSON CONF 内容解析成可自动化处理的 schema
 *
 * @param {Object}  conf 按照约定格式的配置 json 文件
 * @return {Object} schema
 */
function parseConfToSchema() {
    var conf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var properties = conf.schema || {};

    (0, _keys2.default)(properties).forEach(function (key) {
        var item = properties[key];

        if (item.type === 'list') {
            if (item.link && !item.dependence) {
                properties[key].list = conf[item.link];
            } else if (item.dependence) {
                properties[item.dependence].list.forEach(function (depItem) {
                    if (depItem.value === conf.defaults[item.dependence]) {
                        properties[key].list = depItem.subList ? depItem.subList[key] || [] : [];
                    }
                });
            }
        }
    });

    return properties;
}

/**
 * 把约定的 JSON CONF 内容解析成标准的 JSON Schema
 *
 * @param {Object}  conf 按照约定格式的配置 json 文件
 * @return {Object} JSON Schema 的对象
 */
function parseConfToJsonSchema() {
    var conf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    var schemas = conf.schema || {};
    var properties = {};
    var required = [];
    var dependence = {};

    (0, _keys2.default)(schemas).forEach(function (key) {
        var item = schemas[key];

        if (!item.disable) {
            properties[key] = {
                type: item.jsonType || item.type,
                description: item.description
            };

            item.regExp && (properties[key].pattern = item.regExp);
            item.required && required.push(key);
        }
    });

    return {
        type: 'object',
        description: 'lavas scaffold json schema',
        properties,
        required,
        dependence
    };
}

/**
 * 获取元 Schema, 即模板选择的 Schema
 *
 * @return {Object} 元 Schema
 */
exports.getMetaSchema = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee() {
    var meta, metaSchema;
    return _regenerator2.default.wrap(function _callee$(_context) {
        while (1) {
            switch (_context.prev = _context.next) {
                case 0:
                    _context.next = 2;
                    return getMeta();

                case 2:
                    meta = _context.sent;
                    metaSchema = parseConfToSchema(meta);


                    store.set('metaSchema', metaSchema);

                    return _context.abrupt('return', metaSchema);

                case 6:
                case 'end':
                    return _context.stop();
            }
        }
    }, _callee, this);
}));

/**
 * 获取 meta JSON Schema, 用于验证 json 表单
 *
 * @return {Object} 返回的 JSON Schema
 */
exports.getMetaJsonSchema = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2() {
    var meta, metaJsonSchema;
    return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
            switch (_context2.prev = _context2.next) {
                case 0:
                    _context2.next = 2;
                    return getMeta();

                case 2:
                    meta = _context2.sent;
                    metaJsonSchema = parseConfToJsonSchema(meta);


                    store.set('metaJsonSchema', metaJsonSchema);

                    return _context2.abrupt('return', metaJsonSchema);

                case 6:
                case 'end':
                    return _context2.stop();
            }
        }
    }, _callee2, this);
}));

/**
 * 获取 Schema, 用于生成用户输入的表单
 *
 * @param {Object} templateConf 每个模版的 config
 * @return {Object} 返回的 JSON Schema
 */
exports.getSchema = function () {
    var templateConf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return parseConfToSchema(templateConf);
};

/**
 * 获取 JSON schema, 用于验证 JSON 表单
 *
 * @param {Object} templateConf 每个模版的 config
 * @return {Object} 返回的 JSON Schema
 */
exports.getJsonSchema = function () {
    var templateConf = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

    return parseConfToJsonSchema(templateConf);
};