'use strict';

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _stringify = require('babel-runtime/core-js/json/stringify');

var _stringify2 = _interopRequireDefault(_stringify);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

/**
 * 从 git 上 download 代码下来
 *
 * @param  {string} repo       git repo
 * @param  {string} targetPath 存储的目标目录
 * @param  {string} branchName 分支名
 * @return {Promise}           promise 对象
 */
var downloadFromGit = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(repo, targetPath, branchName) {
        var diskCommand, diskName;
        return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
                switch (_context.prev = _context.next) {
                    case 0:
                        if (utils.hasCommand('git')) {
                            _context.next = 2;
                            break;
                        }

                        throw new Error(locals.NO_GIT_COMMAND + ',' + locals.NO_GIT_COMMAND_DESC);

                    case 2:
                        _context.prev = 2;

                        // Fixed: windows 下跨盘 cd 命令，必须先要指定盘名
                        diskCommand = '';


                        if (os.platform() === 'win32') {
                            diskName = targetPath.split(':')[0];

                            if (diskName) {
                                diskCommand = diskName + ': &&';
                            }
                        }

                        // 如果当前文件系统有 download 的缓存，就不重新 clone 了，将代码直接 pull 下来就好了。

                        if (!(fs.existsSync(targetPath) && fs.existsSync(path.resolve(targetPath, '.git')))) {
                            _context.next = 10;
                            break;
                        }

                        _context.next = 8;
                        return exec(`${diskCommand}cd ${targetPath} && git checkout ${branchName} && git pull`);

                    case 8:
                        _context.next = 16;
                        break;

                    case 10:
                        fs.existsSync(targetPath) && fs.removeSync(targetPath);
                        fs.mkdirsSync(targetPath);
                        _context.next = 14;
                        return exec(`git clone ${repo} ${targetPath}`);

                    case 14:
                        _context.next = 16;
                        return exec(`${diskCommand}cd ${targetPath} && git checkout ${branchName}`);

                    case 16:
                        return _context.abrupt('return', targetPath);

                    case 19:
                        _context.prev = 19;
                        _context.t0 = _context['catch'](2);
                        throw new Error(locals.DOWNLOAD_TEMPLATE_ERROR);

                    case 22:
                    case 'end':
                        return _context.stop();
                }
            }
        }, _callee, this, [[2, 19]]);
    }));

    return function downloadFromGit(_x, _x2, _x3) {
        return _ref.apply(this, arguments);
    };
}();

/**
 * 删除某个目录中的指定文件或文件夹
 *
 * @param {string} dir 根目录
 * @param {*} ignores 过滤的文件或文件夹数组
 */


/**
 * 获取模版信息
 *
 * @param  {Object} metaParam 元参数
 * @return {Object} framework 和 template 信息
 */
var getTemplateInfo = function () {
    var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(metaParam) {
        var meta, frameworkValue, templateValue, framework, template, version;
        return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
                switch (_context2.prev = _context2.next) {
                    case 0:
                        _context2.prev = 0;
                        _context2.next = 3;
                        return getMeta();

                    case 3:
                        meta = _context2.sent;
                        frameworkValue = metaParam.framework || meta.defaults.framework || 'vue';
                        templateValue = metaParam.template || meta.defaults.template || 'lavasTemplate';
                        framework = meta.frameworks.filter(function (item) {
                            return item.value === frameworkValue;
                        })[0];
                        template = framework.subList.template.filter(function (item) {
                            return item.value === templateValue;
                        })[0];
                        version = meta.version;


                        store.set('framework', framework);
                        store.set('template', template);
                        store.set('version', version);

                        return _context2.abrupt('return', {
                            framework,
                            template,
                            version
                        });

                    case 15:
                        _context2.prev = 15;
                        _context2.t0 = _context2['catch'](0);
                        throw new Error(locals.META_TEMPLATE_ERROR);

                    case 18:
                    case 'end':
                        return _context2.stop();
                }
            }
        }, _callee2, this, [[0, 15]]);
    }));

    return function getTemplateInfo(_x5) {
        return _ref2.apply(this, arguments);
    };
}();

/**
 * 下载一个指定的模版
 *
 * @param  {Object} metaParams  导出模版所需字段, 从 mataSchema 中得出
 * @return {Objecy}             导出的结果
 */


function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * @file templates modules
 * @author mj(zoumiaojiang@gmail.com)
 */

/* eslint-disable fecs-prefer-async-await */
var fs = require('fs-extra');
var path = require('path');
var os = require('os');
var glob = require('glob');
var archiver = require('archiver');
var etpl = require('etpl');
var Ajv = require('ajv');
var exec = require('mz/child_process').exec;

var conf = require('./config');
var getMeta = require('./getMeta');
var store = require('./store');
var schema = require('./schema');
var utils = require('../utils');
var locals = require('../../locals')();function deleteFilter(dir) {
    var ignores = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : [];

    ignores.concat(...conf.DEFAULT_EXPORTS_IGNORES).forEach(function (target) {
        var targetPath = path.resolve(dir, target);
        fs.existsSync(targetPath) && fs.removeSync(targetPath);
    });
}

/**
 * 给 Lavas 工程指定 package.json 文件
 *
 * @param {string} dir    指定添加 package.json 文件的目录
 * @param {Object} params 渲染的参数
 */
function addPackageJson(dir, params) {
    var templateConfig = store.get('templateConfig');
    var version = store.get('version') || '2';
    var etplCompile = new etpl.Engine(templateConfig.etpl || conf.ETPL);
    var packageJson = templateConfig.exportsPackageJson;

    packageJson.lavas = {
        core: templateConfig.core || 'lavas-core-vue',
        version
    };

    var fileName = 'package.json';
    var filePath = path.resolve(dir, fileName);
    var fileContent = packageJson && typeof packageJson === 'object' ? (0, _stringify2.default)(packageJson, null, 4) : fs.readFileSync(path.resolve(__dirname, 'templates', 'package.json'), 'utf8');

    // 如果没有在模版中指定 package.json 的时候，就需要使用默认的文件了
    params.lavasCoreName = templateConfig.core || 'lavas-core-vue';
    fileContent = etplCompile.compile(fileContent)(params);

    fs.writeFileSync(filePath, fileContent);
}

/**
 * 渲染 template 里面的所有文件
 *
 * @param  {Object} params    收集的用户输入字段
 * @param  {string} tmpStoreDir  临时文件夹存储路径
 * @return {Promise}          渲染 promise
 */
function renderTemplate(params, tmpStoreDir) {
    var templateConfig = store.get('templateConfig');
    var dirPath = params.dirPath || process.cwd();
    var etplCompile = new etpl.Engine(templateConfig.etpl || conf.ETPL);

    // 把指定的开发者不需要的文件和文件夹都删掉
    deleteFilter(tmpStoreDir, templateConfig.exportsIgnores);

    return new _promise2.default(function (resolve, reject) {
        return glob('**/*', {
            cwd: tmpStoreDir,
            ignore: (templateConfig.renderIgnores || []).concat(...conf.DEFAULT_RENDER_IGNORES)
        }, function (err, files) {
            files.forEach(function (file) {
                var filePath = path.resolve(tmpStoreDir, file);

                if (fs.statSync(filePath).isFile()) {
                    var content = fs.readFileSync(filePath, 'utf8');

                    // 这里可以直接通过外界配置的规则，重新计算出一份数据，只要和 template 里面的字段对应上就好了
                    var extDataTpls = templateConfig.extData || {};
                    var extData = {};
                    var commonData = conf.COMMON_DATA;

                    (0, _keys2.default)(extDataTpls).forEach(function (key) {
                        extData[key] = etplCompile.compile(`${extDataTpls[key]}`)(params);
                    });

                    var renderData = (0, _assign2.default)({}, params, extData, commonData);
                    var afterCon = etplCompile.compile(content)(renderData);

                    fs.writeFileSync(filePath, afterCon);
                }
            });

            addPackageJson(tmpStoreDir, params);

            if (params.isStream) {
                var archive = archiver('zip', { zlib: { level: 9 } });
                var tmpZipPath = path.resolve(tmpStoreDir, '..', 'tmp.zip');
                var output = fs.createWriteStream(tmpZipPath);

                archive.pipe(output);
                archive.directory(tmpStoreDir, params.name);
                archive.finalize().on('finish', function () {
                    return resolve(fs.createReadStream(tmpZipPath));
                });
            } else {
                fs.copySync(tmpStoreDir, dirPath);
                resolve(dirPath);
            }
        });
    });
}exports.download = function () {
    var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3() {
        var metaParams = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {};

        var _ref4, framework, template, version, gitRepo, storeDir, branchName, ajv, metaJsonSchema, validate, valid, templateConfigContent, templateConfig;

        return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
                switch (_context3.prev = _context3.next) {
                    case 0:
                        _context3.next = 2;
                        return getTemplateInfo(metaParams);

                    case 2:
                        _ref4 = _context3.sent;
                        framework = _ref4.framework;
                        template = _ref4.template;
                        version = _ref4.version;
                        gitRepo = template.git;
                        storeDir = path.resolve(conf.LOCAL_TEMPLATES_DIR, framework.value, template.value + '_' + version);
                        branchName = template.branch || 'master';
                        ajv = new Ajv({ allErrors: true });
                        _context3.t0 = store.get('metaJsonSchema');

                        if (_context3.t0) {
                            _context3.next = 15;
                            break;
                        }

                        _context3.next = 14;
                        return schema.getMetaJsonSchema();

                    case 14:
                        _context3.t0 = _context3.sent;

                    case 15:
                        metaJsonSchema = _context3.t0;
                        validate = ajv.compile(metaJsonSchema);
                        valid = validate(metaParams);

                        if (valid) {
                            _context3.next = 20;
                            break;
                        }

                        throw new Error((0, _stringify2.default)(validate.errors));

                    case 20:
                        _context3.next = 22;
                        return downloadFromGit(gitRepo, storeDir, branchName);

                    case 22:
                        store.set('storeDir', storeDir);

                        templateConfigContent = fs.readFileSync(path.resolve(storeDir, 'meta.json'), 'utf-8');
                        templateConfig = JSON.parse(templateConfigContent);


                        store.set('templateConfig', templateConfig);

                        return _context3.abrupt('return', templateConfig);

                    case 27:
                    case 'end':
                        return _context3.stop();
                }
            }
        }, _callee3, this);
    }));

    return function () {
        return _ref3.apply(this, arguments);
    };
}();

/**
 * 渲染指定的模板模版
 *
 * @param {Object} params 收集到的用户输入的参数
 * @return {*} 导出的结果
 */
exports.render = function () {
    var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(params) {
        var templateConfig, tmpStoreDir, storeDir, ajv, jsonSchema, validate, valid, renderResult;
        return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
                switch (_context4.prev = _context4.next) {
                    case 0:
                        _context4.t0 = store.get('templateConfig');

                        if (_context4.t0) {
                            _context4.next = 5;
                            break;
                        }

                        _context4.next = 4;
                        return this.download(params);

                    case 4:
                        _context4.t0 = _context4.sent;

                    case 5:
                        templateConfig = _context4.t0;
                        tmpStoreDir = path.resolve(conf.LOCAL_TEMPLATES_DIR, `${Date.now()}`);
                        storeDir = store.get('storeDir');
                        ajv = new Ajv({ allErrors: true });
                        jsonSchema = schema.getJsonSchema(templateConfig);
                        validate = ajv.compile(jsonSchema);
                        valid = validate(params);

                        if (valid) {
                            _context4.next = 14;
                            break;
                        }

                        throw new Error((0, _stringify2.default)(validate.errors));

                    case 14:
                        _context4.prev = 14;

                        if (fs.existsSync(storeDir)) {
                            _context4.next = 18;
                            break;
                        }

                        _context4.next = 18;
                        return this.download(params);

                    case 18:

                        fs.mkdirsSync(tmpStoreDir);
                        fs.copySync(storeDir, tmpStoreDir);

                        _context4.next = 22;
                        return renderTemplate(params, tmpStoreDir);

                    case 22:
                        renderResult = _context4.sent;


                        fs.removeSync(tmpStoreDir);

                        return _context4.abrupt('return', renderResult);

                    case 27:
                        _context4.prev = 27;
                        _context4.t1 = _context4['catch'](14);
                        throw new Error(locals.RENDER_TEMPLATE_ERROR);

                    case 30:
                    case 'end':
                        return _context4.stop();
                }
            }
        }, _callee4, this, [[14, 27]]);
    }));

    return function (_x7) {
        return _ref5.apply(this, arguments);
    };
}();